/*
   SampleApp.cc.prg
   A sample dBL program from 
   The dBASE Book
   by Kenneth J. Mayer. 
   Program copyright 2005-2013 
   Permission granted for use as a learning tool for dBASE Plus.

   This has been updated for dBASE Plus 2.8 and later versions.
   
   Sample from: Chapter 28
   This is the application object used to start/close an application,
   trimmed down from work done on a recent project by myself.

   This has been copied from Chapter 28, and modified as needed to
   show some new aspects of the application from Chapter 26 ...

   Dependencies: Setup.prg
                 MainMenu.mnu
                 :dUFLP:Ini.cc

   To see this work properly, you should download the dUFLP
   from the author's website, install it as designed, and
   then the code should work ... otherwise the custom class
   "ini.cc" will not exist, and will cause problems.
*/
#define APP_VERSION	"1.0"
#define APP_NAME		"A Sample Application"

class SampleApp

   // Set up procedures in memory -- calling the SETUP.PRG file:
   if not setup()
   	release object this
      return
   endif

   // assign the instance of this class
   // as an object reference off the _app object:
   _app.Sample = this 

   /*------------------------------------------------
      This method handles opening the application
   ------------------------------------------------*/
   function open
      // set scope of variables used:
      local cPath, cSection, cEntry

      // set the menu:
      set procedure to MainMenu.mnu additive
      this.rootMenu = new MainMenuMenu( _app.framewin, "Root" )

      // You could insert code to load a toolbar here.

      // Set Shell
      shell( false, true )

      // Save titlebar text:
      this.OldTitleText = _app.Framewin.text

      // set the speedbar/statusbar off
      _app.Speedbar  := false // turn it off
      _app.StatusBar := false // same

      // if you save specific settings into a .INI file,
      // this might be a good place to read them, using
      // a file object, or a .ini class (such as the 
      // one in the dBASE Users' Function Library Project),
      // or code similar to what is used in the chapter
      // of the book dealing with Windows API calls.
      // use .INI file to check for a couple of user preferences:
      set procedure to :dUFLP:INI.cc additive

      // are we in the runtime, or are we
      // in the IDE? 
      // RUNTIME first:
      if ("RUNTIME" $ upper( version(1) ))
         // we are in the runtime, use the default
         // .INI file which should be in the same
         // folder the executable is in:
         _app.ini = new ini( _app.IniFile )

         // ***** IMPORTANT ***** 
         // For a Deployed application, it is best to 
         // put a User BDE Alias in the .INI file for the
         // executable. That is what we are going to do
         // here. So there is no need for an executable
         // to set up the alias each time the application
         // starts. (See Chapter 28 ...)

         // if you wished, you could put a call to the 
         // startup program "InitNewUser.prg" -- see instructions
         // in Chapter 26 about this ...
         // do InitNewUser // for example

      else 
         // otherwise we are in the IDE
         // this needs to be changed to 
         // work with either "MyFish.ini" or
         // "MyFishDEO.ini" for testing in this
         // chapter:
         if "MYFISH.EXE" $ upper( _app.ExeName ) 
             _app.ini = new ini( set("directory")+"\MyFish.ini" )
         else // theoretically this will be MyFishDEO:
             _app.ini = new ini( set("directory")+"\MyFishDEO.ini" )
         endif // check for executable name

         // ***** IMPORTANT ***** 
         // add path to database, and create temporary database alias
         // (User BDE Alias)
         _app.session.addAlias( "DBASEFISH", "DBASE", "PATH:"+;
                                "C:\dBASEBook1_SourceCode\Tables")

      endif // check for Runtime vs. IDE

      // ---------------------------------------------------
      // Note about code shown below -- it was written
      // for an application that allows the user, through the
      // use of a menu option and a dialog form, to set
      // this preference. The menu option does not exist
      // in this smaller sample application. It is here as
      // an example of how it works, how to read from
      // a .INI file using INI.CC, and/or to create the
      // name/value pairs first if needed.
      
      // set speedtip preference:
      cSection = "Preferences"
      cEntry   = "ShowSpeedTips"
      // if it doesn't exist, create it with default:
      if empty( _app.ini.getValue( cSection, cEntry ) )
         _app.ini.setValue( cSection, cEntry, "true" )
      endif
      // get value, and set (convert string to logical), assign 
      // as a custom property of this application object  ...
      this.ShowSpeedTips = ;
           iif( _app.ini.getValue( cSection, cEntry ) == "true",;
                                   true, false )

      /* 
         disable the 'x' for _app ONLY if we're
         in runtime. This forces the user to use
         the "File" menu to exit the application.
         Uses function sysClose below, taken from
         the dUFLP's "MiscAPI.prg" file.
      */
      if "RUNTIME" $ upper( version(1) )
         sysClose( _app.Framewin, false ) // disable
      endif

      // display version and text in titlebar ....
      _app.FrameWin.text := APP_NAME + " -- " + APP_VERSION

      // make sure application is "on top"
      if type('BringWindowToTop') # 'FP'
      	extern clogical BringWindowToTop( cHandle ) user32
      endif
      BringWindowToTop( _app.framewin.hWnd )

   return

   /*------------------------------------------------
      Shutdown
   ------------------------------------------------*/
   function close

      // close anything we may have left open:
      close forms

      // close any open data:
      close tables
      close databases

      // reset titlebar:
      _app.Framewin.text := this.OldTitleText
      this.OldTitleText  := null

      // Save any settings to the application's .ini file
      // such as user preferences here.

      // release menu
      m = this.rootMenu
      release object m
      release m

      // if you loaded a toolbar in the open() method,
      // it would be prudent to detach it here, and release
      // the object reference.

      // release application object
      release object this
      _app.Sample := null

      // restore _app.framewin ...
      shell( true, true )

      // issue 'quit' only if we're in RUNTIME, otherwise
      // testing gets really annoying ...

      // If you don't issue the 'quit', then the shell
      // gets left on screen, which is weird, to say the least.
      if "RUNTIME" $ upper( version(1) )
         quit
      else
         // reset:
         _app.speedbar  := true
         _app.statusbar := true
         sysClose( _app.Framewin, true ) // enable close button
         keyboard('{ctrl+tab}')
      endif
   return

endclass

/* 
   ---------------------------------------------------------------
   Enables or disables the "close" item on the system menu as well 
   as the Win95 X.

      Syntax:    sysClose( oForm, lEnabled )
      Example:   sysClose( _app.FrameWin, false ) // disable
                 sysClose( _app.FrameWin, true )  // ensable

   Posted by Gary White in the VdBASE newsgroups
   7/3/1998

   Returns the previous state of the menu item.
   ---------------------------------------------------------------
*/
function sysClose
parameters oForm, lEnabled
local hMenu, lRetVal
#define MF_ENABLED      0x00000000
#define MF_GRAYED       0x00000001
#define SC_CLOSE        0xF060

   if argCount() # 2 or ;
         type("oForm.hWnd") # "N" ;
         or type("lEnabled") # "L"
      msgbox( "Correct syntax is:  sysClose( oForm, lEnabled )", ;
         "Error", 16 )
      return
   endif
   if empty( oForm.hWnd )
      msgbox( "Form is not open", "Error", 16 )
      return
   endif

   if type( "GetSystemMenu" ) # "FP"
     extern cint GetSystemMenu( cHandle, cInt ) USER32
   endif
   if type( "EnableMenuItem" ) # "FP"
      extern CLOGICAL EnableMenuItem( cHandle, cInt, cInt) user32
   endif
	hMenu = GetSystemMenu( oForm.hWnd, 0 )
   if lEnabled
   	lRetVal = EnableMenuItem( hMenu, SC_CLOSE, MF_ENABLED )
   else
   	lRetVal = EnableMenuItem( hMenu, SC_CLOSE, MF_GRAYED )
	endif
return not lRetVal

