/*
   CascadeDeleteExample.prg
   A sample dBL program from 
   The dBASE Book
   by Kenneth J. Mayer. 
   Program copyright 2012. 
   Permission granted for use as a learning tool for dBASE Plus.

   Sample from: Chapter 8 
   This code will show how Cascade Deletes and Cascade Updates
   can be implemented in your application. As mentioned in the 
   book, Cascade Updates are not generally a good idea, as if
   handled improperly you can generate orphaned child records.
*/

clear

// As this code relies on the data being exactly as needed, 
// we're not asking about tables existing ... running this
// routine once will delete data in two of the tables. This
// will ensure the data is how it shoudl be at the beginning:
do MakeChapter8Tables

// -----------------------------------------------------
// set up queries

// Parent table:
qParent = new query()
qParent.sql := "select * from ParentTest"
qParent.active := true
// short references:
rParent = qParent.rowset
fParent = rParent.fields

// Child table:
qChild = new query()
qChild.sql := "select * from ChildTest"
qChild.active := true
// short references:
rChild = qChild.rowset
fChild = rChild.fields
// set the index:
rChild.indexName := "LinkField"

// to test this, we need to set up the relationship here:
// set the masterRowset/masterFields:
rChild.masterRowset := rParent
rChild.masterFields := "LINKFIELD"

// First we need to set up the call to the
// event handler:
rParent.canDelete = rowset_canDelete

// display information in tables:
? "Data in tables prior to test:"
?
rParent.first()
do while not rParent.endOfSet
   ? fParent["LinkField"].value+" -- "+fParent["ParentData"].value

   // child table:
   rChild.first()
   do while not rChild.endOfSet
      ? "   "+fChild["LinkField"].value+" -- "+fChild["ChildData"].value
      rChild.next()
   enddo

   // navigate in parent table:
   rParent.next()

enddo

?
wait "Press a key to continue test ..."

// to delete a row in the parent table, we 
// need to navigate to the record, then
// try deleting it ... 
rParent.first()

// the first parent record has three children records, 
// so we'll try that one:
?
? "Deleting first record in parent table."
?
rParent.delete()

// display information in tables:
? "Data in tables after delete:"
?
rParent.first()
do while not rParent.endOfSet
   ? fParent["LinkField"].value+" -- "+fParent["ParentData"].value

   // child table:
   rChild.first()
   do while not rChild.endOfSet
      ? "   "+fChild["LinkField"].value+" -- "+fChild["ChildData"].value
      rChild.next()
   enddo

   // navigate in parent table:
   rParent.next()

enddo


// --------------------------------------------------------------------
// Cleanup
// --------------------------------------------------------------------
qParent.active := false
release object qParent
qParent := null

qChild.active := false
release object qChild
qChild := null
// end of program

// --------------------------------------------------------------------
// Event Handler needed to make this work:
// --------------------------------------------------------------------

function rowset_canDelete
   r = rChild // we could probably pass this to the 
              // event handler, but ...

   // loop until we get to the end of the child rowset -
   // don't panic - the child rowset is constrained
   // by the key field in the parent rowset, and only
   // the child rows that match the current parent
   // are seen here:
   do while not r.endOfSet
      // delete it:
      r.delete()
   enddo
   // that's it - the child rows are deleted, when
   // this method is done, the next event to fire
   // is the delete on the parent, and then if you
   // had an onDelete call, that would fire ...

return true



